package cn.iocoder.yudao.module.bpm.dal.mysql.task;

import cn.iocoder.yudao.module.bpm.controller.admin.task.vo.instance.BpmProcessInstanceMyPageReqVO;
import cn.iocoder.yudao.module.bpm.dal.dataobject.task.BpmProcessInstanceExtDO;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.mybatis.core.mapper.BaseMapperX;
import cn.iocoder.yudao.framework.mybatis.core.query.QueryWrapperX;
import cn.iocoder.yudao.module.order.dal.dataobject.order.OrderDO;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import org.apache.ibatis.annotations.Mapper;
import org.apache.ibatis.annotations.Param;
import org.apache.ibatis.annotations.ResultType;
import org.apache.ibatis.annotations.Select;

import java.util.List;

@Mapper
public interface BpmProcessInstanceExtMapper extends BaseMapperX<BpmProcessInstanceExtDO> {

    default PageResult<BpmProcessInstanceExtDO> selectPage(Long userId, BpmProcessInstanceMyPageReqVO reqVO) {
        return selectPage(reqVO, new QueryWrapperX<BpmProcessInstanceExtDO>()
                .eqIfPresent("start_user_id", userId)
                .likeIfPresent("name", reqVO.getName())
                .eqIfPresent("process_definition_id", reqVO.getProcessDefinitionId())
                .eqIfPresent("category", reqVO.getCategory())
                .eqIfPresent("status", reqVO.getStatus())
                .eqIfPresent("result", reqVO.getResult())
                .betweenIfPresent("create_time", reqVO.getBeginCreateTime(), reqVO.getEndCreateTime())
                .orderByDesc("id"));
    }

    default BpmProcessInstanceExtDO selectByProcessInstanceId(String processDefinitionId) {
        return selectOne("process_instance_id", processDefinitionId);
    }

    default void updateByProcessInstanceId(BpmProcessInstanceExtDO updateObj) {
        update(updateObj, new QueryWrapper<BpmProcessInstanceExtDO>()
                .eq("process_instance_id", updateObj.getProcessInstanceId()));
    }


    @ResultType(BpmProcessInstanceExtDO.class)
    @Select({
            "<script>",
            "SELECT",
            "id,",
            "start_user_id,",
            "name,",
            "process_instance_id,",
            "process_definition_id,",
            "category,",
            "STATUS,",
            "result,",
            "end_time,",
            "form_variables ->> '$.businessNo' as business_no,",
            "create_time,",
            "update_time,",
            "creator,",
            "updater,",
            "deleted ",
            "FROM",
            "bpm_process_instance_ext ",
            "WHERE",
            "deleted = 0 ",
            "<when test = 'userId != null '>",
            "and start_user_id = #{userId}",
            "</when>",
            "<when test = 'reqVO.category != null '>",
            "and category = #{reqVO.category}",
            "</when>",
            "<when test = 'reqVO.status != null '>",
            "and status = #{reqVO.status}",
            "</when>",
            "<when test = 'reqVO.result != null '>",
            "and result = #{reqVO.result}",
            "</when>",
            "<when test = 'reqVO.beginCreateTime != null and reqVO.endCreateTime != null '>",
            "and create_time between #{reqVO.beginCreateTime} and #{reqVO.endCreateTime}",
            "</when>",
            "<when test = 'reqVO.name != null and reqVO.name != \"\" '>",
            "and name LIKE concat('%',#{reqVO.name},'%')",
            "</when>",
            "<when test = 'reqVO.businessNo != null and reqVO.businessNo != \"\" '>",
            "and form_variables ->> '$.businessNo' LIKE concat('%',#{reqVO.businessNo},'%')",
            "</when>",
            " order by id desc ",
            "</script>"
    })
    IPage<BpmProcessInstanceExtDO> getMyProcessInstancePage(@Param("mpPage") IPage<BpmProcessInstanceExtDO> mpPage, @Param("userId") Long userId, @Param("reqVO") BpmProcessInstanceMyPageReqVO reqVO);

}
