package cn.iocoder.yudao.module.bpm.service;

import cn.hutool.core.util.StrUtil;
import cn.iocoder.yudao.framework.common.exception.ServiceException;
import cn.iocoder.yudao.module.bpm.api.BpmCreateServiceFactory;
import cn.iocoder.yudao.module.bpm.api.task.BpmProcessInstanceApi;
import cn.iocoder.yudao.module.bpm.api.task.dto.BpmProcessInstanceCreateReqDTO;
import cn.iocoder.yudao.module.bpm.resp.TaskResp;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.Map;

import static cn.iocoder.yudao.module.bpm.enums.ErrorCodeConstants.PROCESS_DESIGNATEDREVIEW_NOT_EMPTY;

/**
 * 流程创建工厂
 */
@Service
public class BpmCreateServiceFactoryImpl implements BpmCreateServiceFactory {

    @Resource
    private BpmProcessInstanceApi processInstanceApi;


    /**
     * 用法是表单对象叫一定要创建一个字段 process_instance_id，用来存储流程创建后返回的ID，字段名称不能变
     *
     * @param userId      用户ID
     * @param orderFormId 表单ID
     * @param workFlowKey WorkFlowEmus枚举下的KEY--创建哪个流程
     * @param businessNo  业务单号
     * @param copyUserId  抄送人列表,不需要传null
     * @return 流程实例ID，需回写到表单对象中
     */
    public String createBmp(Long userId, Long orderFormId, String workFlowKey,
                            String businessNo, String[] copyUserId) {
        Map<String, Object> processInstanceVariables = new HashMap<>();
        if (StrUtil.isNotBlank(businessNo)) {
            processInstanceVariables.put("businessNo", businessNo);
        }
        String processInstanceId =
                processInstanceApi.createProcessInstance(userId,
                        new BpmProcessInstanceCreateReqDTO().setProcessDefinitionKey(workFlowKey).setVariables(processInstanceVariables).setBusinessKey(String.valueOf(orderFormId)).setCopyUserIds(copyUserId));
        return processInstanceId;
    }

    /**
     * 用法是表单对象叫一定要创建一个字段 process_instance_id，用来存储流程创建后返回的ID，字段名称不能变
     *
     * @param userId           用户ID
     * @param orderFormId      表单ID
     * @param workFlowKey      WorkFlowEmus枚举下的KEY--创建哪个流程
     * @param businessNo       业务单号
     * @param copyUserId       抄送人列表,不需要传null
     * @param designatedReview 指定审核人
     * @return 流程实例ID，需回写到表单对象中
     */
    public String createBmpDesignated(Long userId, Long orderFormId, String workFlowKey, String businessNo, String[] copyUserId, String designatedReview) {
        Map<String, Object> processInstanceVariables = new HashMap<>();
        if (StrUtil.isNotBlank(businessNo)) {
            processInstanceVariables.put("businessNo", businessNo);
        }
        if (StrUtil.isBlank(designatedReview)) {
            throw new ServiceException(PROCESS_DESIGNATEDREVIEW_NOT_EMPTY);
        }
        processInstanceVariables.put("designatedReview", designatedReview);
        String processInstanceId = processInstanceApi.createProcessInstance(userId, new BpmProcessInstanceCreateReqDTO().setProcessDefinitionKey(workFlowKey).setVariables(processInstanceVariables).setBusinessKey(String.valueOf(orderFormId)).setCopyUserIds(copyUserId));
        return processInstanceId;
    }

    @Override
    public void cancelBmpById(Long userId, String bpmProcessInstanceId, String reason) {
        processInstanceApi.cancelBmpById(userId, bpmProcessInstanceId, reason);
    }

    @Override
    public TaskResp getLastTaskByProcessInstanceId(String processInstanceId) {
        return processInstanceApi.getLastTaskByProcessInstanceId(processInstanceId);
    }
}
