package cn.iocoder.yudao.module.bpm.service.message;

import cn.iocoder.yudao.framework.web.config.WebProperties;
import cn.iocoder.yudao.module.bpm.service.message.dto.BpmMessageSendWhenProcessInstanceApproveReqDTO;
import cn.iocoder.yudao.module.bpm.service.message.dto.BpmMessageSendWhenProcessInstanceRejectReqDTO;
import cn.iocoder.yudao.module.bpm.service.message.dto.BpmMessageSendWhenTaskCreatedReqDTO;
import cn.iocoder.yudao.module.ecw.api.internalMessage.InternalMessageApi;
import cn.iocoder.yudao.module.ecw.api.internalMessage.dto.InternalMessageCreateDto;
import cn.iocoder.yudao.module.system.api.sms.SmsSendApi;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;
import org.springframework.validation.annotation.Validated;

import javax.annotation.Resource;
import java.util.Arrays;
import java.util.Date;

/**
 * BPM 消息 Service 实现类
 *
 * @author 捷道源码
 */
@Service
@Validated
@Slf4j
public class BpmMessageServiceImpl implements BpmMessageService {

    @Resource
    private SmsSendApi smsSendApi;


    @Resource
    InternalMessageApi internalMessageApi;

    @Resource
    private WebProperties webProperties;

    @Override
    @Async
    public void sendMessageWhenProcessInstanceApprove(BpmMessageSendWhenProcessInstanceApproveReqDTO reqDTO) {
        /*Map<String, Object> templateParams = new HashMap<>();
        templateParams.put("processInstanceName", reqDTO.getProcessInstanceName());
        templateParams.put("detailUrl", getProcessInstanceDetailUrl(reqDTO.getProcessInstanceId()));*/
        InternalMessageCreateDto dto = InternalMessageCreateDto.builder().toIdList(Arrays.asList(reqDTO.getStartUserId()))
                .fromId("系统消息")
                .sendTime(new Date())
                .title(reqDTO.getTitle())
                .content(reqDTO.getComment())
                .type(reqDTO.getType()).build();
        internalMessageApi.createInternalMessage(dto);
      /*  smsSendApi.sendSingleSmsToAdmin(BpmMessageConvert.INSTANCE.convert(reqDTO.getStartUserId(),
                BpmMessageEnum.PROCESS_INSTANCE_APPROVE.getSmsTemplateCode(), templateParams));*/
    }

    @Override
    @Async
    public void sendMessageWhenProcessInstanceReject(BpmMessageSendWhenProcessInstanceRejectReqDTO reqDTO) {
       /* Map<String, Object> templateParams = new HashMap<>();
        templateParams.put("processInstanceName", reqDTO.getProcessInstanceName());
        templateParams.put("comment", reqDTO.getComment());
        templateParams.put("detailUrl", getProcessInstanceDetailUrl(reqDTO.getProcessInstanceId()));*/
        InternalMessageCreateDto dto = InternalMessageCreateDto.builder().toIdList(Arrays.asList(reqDTO.getStartUserId()))
                .fromId("系统消息")
                .sendTime(new Date())
                .title(reqDTO.getTitle())
                .content(reqDTO.getComment())
                .type(reqDTO.getType()).build();
        internalMessageApi.createInternalMessage(dto);
        /*smsSendApi.sendSingleSmsToAdmin(BpmMessageConvert.INSTANCE.convert(reqDTO.getStartUserId(),
                BpmMessageEnum.PROCESS_INSTANCE_REJECT.getSmsTemplateCode(), templateParams));*/
    }

    @Override
    public void sendMessageWhenTaskAssigned(BpmMessageSendWhenTaskCreatedReqDTO reqDTO) {
        /*Map<String, Object> templateParams = new HashMap<>();
        templateParams.put("processInstanceName", reqDTO.getProcessInstanceName());
        templateParams.put("taskName", reqDTO.getTaskName());
        templateParams.put("startUserNickname", reqDTO.getStartUserNickname());
        templateParams.put("detailUrl", getProcessInstanceDetailUrl(reqDTO.getProcessInstanceId()));*/
        InternalMessageCreateDto dto = InternalMessageCreateDto.builder().toIdList(Arrays.asList(reqDTO.getStartUserId()))
                .fromId("系统消息")
                .sendTime(new Date())
                .title(reqDTO.getProcessInstanceName())
                .content("您有一条新的流程待审批-<a href=\"" + getProcessInstanceDetailUrl(reqDTO.getProcessInstanceId()) + "\">" + reqDTO.getProcessInstanceName() + "</a>")
                .type(reqDTO.getType()).build();
        internalMessageApi.createInternalMessage(dto);
        /*smsSendApi.sendSingleSmsToAdmin(BpmMessageConvert.INSTANCE.convert(reqDTO.getStartUserId(),
                BpmMessageEnum.TASK_ASSIGNED.getSmsTemplateCode(), templateParams));*/
    }

    private String getProcessInstanceDetailUrl(String taskId) {
        return webProperties.getAdminUi().getUrl() + "/bpm/process-instance/detail?id=" + taskId;
    }

}
